;Installer for additional Gimp plugins
;public domain by Jernej Simoncic <jernej.simoncic@isg.si>

#pragma singlequotes(1)

;minimum gimp version
#define minversion='2.0.0'

;#define dontcompress

[Setup]
AppName=GIMP Animation Package
;if AppID is the same as in the Gimp's installer, the plug-ins will be uninstalled together with Gimp
;Note that in this case, you won't be able to uninstall the plug-in separately from Gimp, and that Gimp
;uninstaller will report that it's uninstalling whatever you put in the AppVerName here
AppID=WinGimp-2.0
;if you use custom AppID, comment out the next line
CreateUninstallRegKey=no
AppVerName=GIMP Animation Package 2.0.0
AppPublisherURL=http://www.gimp.org/win32/
AppSupportURL=http://groups.yahoo.com/group/gimpwin-users/
AppUpdatesURL=http://www2.arnes.si/~sopjsimo/gimp/
DefaultDirName={code:GetGimpDir|null}
DefaultGroupName={code:GetGimpGroup|null}
DisableProgramGroupPage=yes
DisableDirPage=yes
FlatComponentsList=yes
;LicenseFile=gpl.rtf
;InfoBeforeFile=faq.rtf
;ChangesAssociations=true
WizardImageFile=big.bmp
WizardImageBackColor=$ffffff
WizardSmallImageFile=small.bmp
WizardImageStretch=no
#ifdef dontcompress
UseSetupLdr=no
OutputDir=_Output\47z
Compression=none
InternalCompressLevel=0
#else
OutputDir=_Output
Compression=lzma/ultra
InternalCompressLevel=ultra
SolidCompression=yes
OutputBaseFilename=gimp-gap-2.0.0-setup
#endif
;AdminPrivilegesRequired=true
MinVersion=4,4
;WizardDebug=yes

[Messages]
WelcomeLabel1=Welcome to the GIMP Animation Package Setup Wizard

;[Types]
;Name: full; Description: Full
;Name: normal; Description: Normal
;Name: custom; Description: Custom; Flags: IsCustom

;[Components]
;Name: "base"; Description: "GIMP-Help"; Types: full normal custom; Flags: fixed
;Name: "int"; Description: "Translations (French, German, Swedish)"; Types: full custom

[Files]
Source: "gap-readme.txt"; DestDir: "{app}"; Flags: isreadme
Source: "lib\*.*"; DestDir: "{app}\lib"; Flags: recursesubdirs comparetimestamp
Source: "share\*.*"; DestDir: "{app}\share"; Flags: recursesubdirs comparetimestamp
;Source: "inthelp\*.*"; DestDir: "{app}\share\gimp\2.0\help"; Flags: recursesubdirs comparetimestamp; Components: int

[Icons]
Name: "{group}\GIMP Animation Package readme"; Filename: "{app}\gap-readme.txt"

[Code]
const
  RegPath = 'SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\WinGimp-2.0_is1';
  RegKey = 'Inno Setup: App Path';
  GrpKey = 'Inno Setup: Icon Group';
  VerKey = 'DisplayName';

var
  GimpPath: String;
  GimpVer: String;
  GimpGroup: String;

function InitializeSetup(): Boolean;
begin
  Result := True;
  GimpPath := '';
  If not RegQueryStringValue(HKLM, RegPath, RegKey, GimpPath) then //find Gimp
    if not RegQueryStringValue(HKCU, RegPath, RegKey, GimpPath) then         //install dir
    begin
      Result := False;
      MsgBox('Gimp 2.0 installation path could not be read from the Registry. Please install Gimp {#=minversion} or later first.', mbError, 0);
      exit;
    end;

  If not RegQueryStringValue(HKLM, RegPath, VerKey, GimpVer) then //find Gimp
    if not RegQueryStringValue(HKCU, RegPath, VerKey, GimpVer) then         //install dir
    begin
    	Result := False;
      MsgBox('Gimp 2.0 version could not be read from the Registry. Please install Gimp {#=minversion} or later first.', mbError, 0);
      exit;
    end;
    
  If not RegQueryStringValue(HKLM, RegPath, GrpKey, GimpGroup) then
    if not RegQueryStringValue(HKCU, RegPath, GrpKey, GimpGroup) then
      MsgBox('Gimp 2.0 program group could not be read from the Registry.', mbInformation, 0);
    
 	if GimpVer < 'The GIMP {#=minversion}' then
 	begin
  	Result := False;
    MsgBox(GimpVer + ' was found installed. To instell these plugins, you need Gimp {#=minversion} or later.', mbError, 0);
    exit;
  end;

end;

function GetGimpDir(S: String): String;
begin
	Result := GimpPath;
end;

function GetGimpGroup(S: String): String;
begin
	Result := GimpGroup;
end;

function UpdateReadyMemo(Space, NewLine, MemoUserInfo, MemoDirInfo, MemoTypeInfo, MemoComponentsInfo, MemoGroupInfo, MemoTasksInfo: String): String;
var S: String;
    ShowAssoc: Boolean;
begin

  (* Fill the 'Ready Memo' with setup information *)

  S := 'Gimp 2.0 was found in:' + NewLine + Space + GetGimpDir('') + NewLine + NewLine;                //installation directory

  S := S + MemoTypeInfo + NewLine + NewLine;           //installation type (full/compact/custom)

  S := S + MemoComponentsInfo + NewLine + NewLine;     //selected components

  Result := S;
end;

