;.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,
;                                                                       ;
;Copyright (c) 2002-2004 Jernej Simoncic                                ;
;                                                                       ;
;This software is provided 'as-is', without any express or implied      ;
;warranty. In no event will the authors be held liable for any damages  ;
;arising from the use of this software.                                 ;
;                                                                       ;
;Permission is granted to anyone to use this software for any purpose,  ;
;including commercial applications, and to alter it and redistribute it ;
;freely, subject to the following restrictions:                         ;
;                                                                       ;
;   1. The origin of this software must not be misrepresented; you must ;
;      not claim that you wrote the original software. If you use this  ;
;      software in a product, an acknowledgment in the product          ;
;      documentation would be appreciated but is not required.          ;
;                                                                       ;
;   2. Altered source versions must be plainly marked as such, and must ;
;      not be misrepresented as being the original software.            ;
;                                                                       ;
;   3. This notice may not be removed or altered from any source        ;
;      distribution.                                                    ;
;.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.,.;
;
;The GIMP setup script, for Inno Setup + ISPP
;extract all gimp's files in a folder (keeping the directory structure), and run the script from that
;folder
;
;other required files (in _install subdir):
; - gpl.rtf - GNU General Public Licence
; - logo.bmp - the image displayed on the first and last wizard pages
; - wilber.bmp - image displayed in the upper-left corner of the wizard
; - default.isl+ - additional strings used in the setup
;
;revision history:
;20040327 - new warnings page, new file associations page, redesigned ready memo,
;           prepared for localization
;20040324 - Gimp 2.0 final
;20040202 - fixed detection if admin installed GTK+2
;20040111 - moved some dependencies to GTK+ 2 setup
;20030905 - adapted Gimp 1.2 install script for Gimp 1.3
;
;TODO:
;- CPU detection on Win95 (should I bother?)
;- remove all hardcoded version numbers
;- clean up the code
;- use IS 4.2.1+ native support for custom translation strings

#pragma singlequotes(1)

;if source was checked out from CVS, which date
;#define cvsdate='20031031'

;gimp version
#define vermajor='2'
#define verminor='0'
#define vermicro='1'
;installer revision (don't set it for the first revision; 2nd revision is 1 etc.)
#define revision='1'

;don't use any compression
#define dontcompress
;when defined, no files will be included in the setup
;#define testing

;components to include
#define include_ft
;#define include_help
;#define include_gap


#define version=vermajor+'.'+verminor+'.'+vermicro
#ifdef cvsdate
  #define gimpver=version+'-CVS-'+cvsdate
#else
  #define gimpver=version
#endif


[Setup]
AppName=The GIMP
;please don't change
AppID=WinGimp-{#= vermajor}.0
AppVerName=The GIMP {#= gimpver}
AppPublisher=<Gimp for Windows homepage>
AppPublisherURL=http://www.gimp.org/win32/
AppSupportURL=http://groups.yahoo.com/group/gimpwin-users/
AppUpdatesURL=http://www2.arnes.si/~sopjsimo/gimp/
DefaultDirName={pf}\GIMP-{#= vermajor}.0
DefaultGroupName=GIMP
AllowNoIcons=true
FlatComponentsList=yes
LicenseFile=_install\gpl.rtf
;InfoBeforeFile=_install\gpl.rtf
ChangesAssociations=true
WizardImageFile=_install\2.0.bmp
WizardImageBackColor=$f3f3f3
WizardSmallImageFile=_install\wilber.bmp
UninstallDisplayIcon={app}\bin\gimp-2.0.exe
#ifdef dontcompress
UseSetupLdr=no
OutputDir=_Output\47z
Compression=none
InternalCompressLevel=0
#else
OutputDir=_Output
Compression=lzma/ultra
InternalCompressLevel=ultra
SolidCompression=yes
OutputBaseFileName=gimp-{#= gimpver}-setup
#endif
;MinVersion=4,4
;WizardDebug=yes

[Tasks]
Name: desktopicon; Description: "{code:GetText|AdditionalIconsDesktop}"; GroupDescription: "{code:GetText|AdditionalIcons}"; MinVersion: 4,4
Name: quicklaunchicon; Description: "{code:GetText|AdditionalIconsQuickLaunch}"; GroupDescription: "{code:GetText|AdditionalIcons}"; MinVersion: 4,4; Flags: unchecked
;Name: gtkrc; Description: "Make The Gimp appear in the current Windows color scheme"; GroupDescription: "Interface"

[Types]
Name: normal; Description: "{code:GetText|TypeTypical}"
Name: full; Description: "{code:GetText|TypeFull}"
Name: compact; Description: "{code:GetText|TypeCompact}"
Name: custom; Description: "{code:GetText|TypeCustom}"; Flags: iscustom

[Components]
Name: base; Description: "{code:GetText|ComponentsBase}"; Types: full compact normal custom; Flags: fixed
Name: loc; Description: "{code:GetText|ComponentsTranslations}"; Types: full custom
#ifdef include_ft
Name: ft; Description: "{code:GetText|ComponentsFreeType}"; Types: full custom
#endif
#ifdef include_help
Name: help; Description: "{code:GetText|ComponentsHelp}"; Types: full custom normal
#endif
#ifdef include_gap
Name: gap; Description: "{code:GetText|ComponentsGAP}"; Types: full custom
#endif

[Files]
Source: "_install\setup.ini"; Flags: dontcopy
Source: "_install\*.isl+"; Flags: dontcopy
#ifndef testing
;GIMP base
Source: "bin\*.exe"; DestDir: "{app}\bin"; Components: base; Flags: recursesubdirs
Source: "etc\*.*"; DestDir: "{app}\etc"; Components: base; Flags: recursesubdirs
;Source: "lib\*.*"; DestDir: "{code:GetGTkDir|none}\lib"
Source: "bin\*.dll"; DestDir: "{app}\bin"; Components: base; Flags: recursesubdirs

;Source: "{code:GetGTkDir|none}\libpng12.dll"; DestDir: "{code:GetGTkDir|none}"; DestName: "libpng.dll"; Components: base; Flags: external onlyifdoesntexist
;Source: "{code:GetGTkDir|none}\jpeg62.dll"; DestDir: "{code:GetGTkDir|none}"; DestName: "jpeg-62.dll"; Components: base; Flags: external onlyifdoesntexist
;Source: "{code:GetGTkDir|none}\zlib1.dll"; DestDir: "{code:GetGTkDir|none}"; DestName: "zlib-1.dll"; Components: base; Flags: external onlyifdoesntexist
;Source: "{code:GetGTkDir|none}\freetype6.dll"; DestDir: "{code:GetGTkDir|none}"; DestName: "freetype-6.dll"; Components: base; Flags: external onlyifdoesntexist
;Source: "bin\*.dll"; DestDir: "{code:GetGTkDir|none}"; Components: base; Flags: recursesubdirs
;Source: "fonts\*.*"; DestDir: "{code:GetGTkDir|none}\..\etc\fonts"; Components: base; Flags: recursesubdirs

Source: "share\*.*"; DestDir: "{app}\share"; Components: base; Flags: recursesubdirs

Source: "lib\gimp\*.env"; DestDir: "{app}\lib\gimp"; Components: base; Flags: recursesubdirs
Source: "lib\gimp\*.dll"; DestDir: "{app}\lib\gimp"; Components: base; Flags: recursesubdirs
Source: "lib\gimp\*.exe"; DestDir: "{app}\lib\gimp"; Components: base; Flags: recursesubdirs

#ifdef include_help
Source: "_components\help\help\*.*"; DestDir: "{app}\share\gimp\2.0\help"; Components: help; Flags: recursesubdirs
Source: "_components\help\inthelp\*.*"; DestDir: "{app}\share\gimp\2.0\help"; Components: help; Flags: recursesubdirs; Check: ComponentCheck(loc)
#endif

#ifdef include_ft
Source: "_components\gimp-freetype\lib\gimp\*.*"; DestDir: "{app}\lib\gimp"; Components: ft; Flags: recursesubdirs
Source: "_components\gimp-freetype\share\*.*"; DestDir: "{app}\share"; Components: ft; Flags: recursesubdirs; Check: ComponentCheck(help)
Source: "_components\gimp-freetype\lib\locale\*.*"; DestDir: "{app}\lib\locale"; Components: ft; Flags: recursesubdirs; Check: ComponentCheck(loc)
#endif

#ifdef include_gap
Source: "_components\gimp-gap-2.0pre1\lib\gimp\*.*"; DestDir: "{app}\lib\gimp"; Components: gap; Flags: recursesubdirs
Source: "_components\gimp-gap-2.0pre1\lib\locale\*.*"; DestDir: "{app}\share"; Components: gap; Flags: recursesubdirs; Check: ComponentCheck(loc)
Source: "_components\gimp-gap-2.0pre1\README.gap.txt"; DestDir: "{app}"; Components: gap; Flags: isreadme
#endif

;GIMP localization
Source: "lib\locale\*.*"; DestDir: "{app}\lib\locale"; Components: loc; Flags: recursesubdirs
#if 0
;TODO
Source: "lib\locale\ca\*.*"; DestDir: "{app}\lib\locale\ca"; Flags: ignoreversion recursesubdirs; Components: loc\ca
Source: "lib\locale\cs\*.*"; DestDir: "{app}\lib\locale\cs"; Flags: ignoreversion recursesubdirs; Components: loc\cs
Source: "lib\locale\da\*.*"; DestDir: "{app}\lib\locale\da"; Flags: ignoreversion recursesubdirs; Components: loc\da
Source: "lib\locale\de\*.*"; DestDir: "{app}\lib\locale\de"; Flags: ignoreversion recursesubdirs; Components: loc\de
Source: "lib\locale\el\*.*"; DestDir: "{app}\lib\locale\el"; Flags: ignoreversion recursesubdirs; Components: loc\el
Source: "lib\locale\en_GB\*.*"; DestDir: "{app}\lib\locale\en_GB"; Flags: ignoreversion recursesubdirs; Components: loc\en_GB
Source: "lib\locale\es\*.*"; DestDir: "{app}\lib\locale\es"; Flags: ignoreversion recursesubdirs; Components: loc\es
Source: "lib\locale\fi\*.*"; DestDir: "{app}\lib\locale\fi"; Flags: ignoreversion recursesubdirs; Components: loc\fi
Source: "lib\locale\fr\*.*"; DestDir: "{app}\lib\locale\fr"; Flags: ignoreversion recursesubdirs; Components: loc\fr
Source: "lib\locale\ga\*.*"; DestDir: "{app}\lib\locale\ga"; Flags: ignoreversion recursesubdirs; Components: loc\ga
Source: "lib\locale\gl\*.*"; DestDir: "{app}\lib\locale\gl"; Flags: ignoreversion recursesubdirs; Components: loc\gl
Source: "lib\locale\hr\*.*"; DestDir: "{app}\lib\locale\hr"; Flags: ignoreversion recursesubdirs; Components: loc\hr
Source: "lib\locale\hu\*.*"; DestDir: "{app}\lib\locale\hu"; Flags: ignoreversion recursesubdirs; Components: loc\hu
Source: "lib\locale\it\*.*"; DestDir: "{app}\lib\locale\it"; Flags: ignoreversion recursesubdirs; Components: loc\it
Source: "lib\locale\ja\*.*"; DestDir: "{app}\lib\locale\ja"; Flags: ignoreversion recursesubdirs; Components: loc\ja
Source: "lib\locale\ko\*.*"; DestDir: "{app}\lib\locale\ko"; Flags: ignoreversion recursesubdirs; Components: loc\ko
Source: "lib\locale\lt\*.*"; DestDir: "{app}\lib\locale\lt"; Flags: ignoreversion recursesubdirs; Components: loc\lt
Source: "lib\locale\nl\*.*"; DestDir: "{app}\lib\locale\nl"; Flags: ignoreversion recursesubdirs; Components: loc\nl
Source: "lib\locale\nn\*.*"; DestDir: "{app}\lib\locale\nn"; Flags: ignoreversion recursesubdirs; Components: loc\nn
Source: "lib\locale\no\*.*"; DestDir: "{app}\lib\locale\no"; Flags: ignoreversion recursesubdirs; Components: loc\no
Source: "lib\locale\pl\*.*"; DestDir: "{app}\lib\locale\pl"; Flags: ignoreversion recursesubdirs; Components: loc\pl
Source: "lib\locale\pt\*.*"; DestDir: "{app}\lib\locale\pt"; Flags: ignoreversion recursesubdirs; Components: loc\pt
Source: "lib\locale\pt_BR\*.*"; DestDir: "{app}\lib\locale\pt_BR"; Flags: ignoreversion recursesubdirs; Components: loc\pt_BR
Source: "lib\locale\ro\*.*"; DestDir: "{app}\lib\locale\ro"; Flags: ignoreversion recursesubdirs; Components: loc\ro
Source: "lib\locale\ru\*.*"; DestDir: "{app}\lib\locale\ru"; Flags: ignoreversion recursesubdirs; Components: loc\ru
Source: "lib\locale\sk\*.*"; DestDir: "{app}\lib\locale\sk"; Flags: ignoreversion recursesubdirs; Components: loc\sk
Source: "lib\locale\sl\*.*"; DestDir: "{app}\lib\locale\sl"; Flags: ignoreversion recursesubdirs; Components: loc\sl
Source: "lib\locale\sv\*.*"; DestDir: "{app}\lib\locale\sv"; Flags: ignoreversion recursesubdirs; Components: loc\sv
Source: "lib\locale\ta\*.*"; DestDir: "{app}\lib\locale\ta"; Flags: ignoreversion recursesubdirs; Components: loc\ta
Source: "lib\locale\tr\*.*"; DestDir: "{app}\lib\locale\tr"; Flags: ignoreversion recursesubdirs; Components: loc\tr
Source: "lib\locale\uk\*.*"; DestDir: "{app}\lib\locale\uk"; Flags: ignoreversion recursesubdirs; Components: loc\uk
Source: "lib\locale\zh_CN\*.*"; DestDir: "{app}\lib\locale\zh_CN"; Flags: ignoreversion recursesubdirs; Components: loc\zh_CN
Source: "lib\locale\zh_TW\*.*"; DestDir: "{app}\lib\locale\zh_TW"; Flags: ignoreversion recursesubdirs; Components: loc\zh_TW
#endif
#endif

[INI]
;URL to Gimp-Win homepage
Filename: "{app}\gimp.url"; Section: "InternetShortcut"; Key: "URL"; String: "http://www.gimp.org/win32/"
;Filename: "{app}\gimpwinfaq.url"; Section: "InternetShortcut"; Key: "URL"; String: "http://wingimpfaq.cjb.net/"

[Registry]
;file associations main key
Root: "HKCR"; Subkey: "TheGIMP20"; ValueType: string; ValueName: ; ValueData: "{code:GetText|GimpImage}"; Flags: uninsdeletekey; Check: AdminCheck(yes)
Root: "HKCR"; Subkey: "TheGIMP20\DefaultIcon"; ValueType: string; ValueName: ; ValueData: "{app}\bin\gimp-2.0.exe,0"; Check: AdminCheck(yes)
#ifexist 'bin\\gimp-win-remote.exe'
Root: "HKCR"; Subkey: "TheGIMP20\shell\open\command"; ValueType: string; ValueName: ""; ValueData: """{app}\bin\gimp-win-remote.exe"" ""%1"""; Check: AdminCheck(yes)
#else
Root: "HKCR"; Subkey: "TheGIMP20\shell\open\command"; ValueType: string; ValueName: ""; ValueData: """{app}\bin\gimp-2.0.exe"" ""%1"""; Check: AdminCheck(yes)
#endif
;file types
Root: "HKCR"; Subkey: ".XCF"; ValueType: string; ValueName: ""; ValueData: "TheGIMP20"; Flags: uninsdeletevalue; Check: AdminCheck(yes)
Root: "HKCR"; Subkey: ".PAT"; ValueType: string; ValueName: ""; ValueData: "TheGIMP20"; Flags: uninsdeletevalue; Check: AdminCheck(yes)
Root: "HKCR"; Subkey: ".GIH"; ValueType: string; ValueName: ""; ValueData: "TheGIMP20"; Flags: uninsdeletevalue; Check: AdminCheck(yes)
Root: "HKCR"; Subkey: ".GBR"; ValueType: string; ValueName: ""; ValueData: "TheGIMP20"; Flags: uninsdeletevalue; Check: AdminCheck(yes)
;check the [Code] section!!
Root: "HKCR"; Subkey: "none"; ValueType: string; ValueName: ""; ValueData: "TheGIMP20"; Flags: uninsdeletevalue; Check: AssociateFiles

;path
Root: "HKLM"; SubKey: "Software\Microsoft\Windows\CurrentVersion\App Paths\gimp-2.0.exe"; ValueType: string; ValueName: ""; ValueData: "{app}\bin\gimp-2.0.exe"; Check: AdminCheck(yes)
Root: "HKLM"; SubKey: "Software\Microsoft\Windows\CurrentVersion\App Paths\gimp-2.0.exe"; ValueType: string; ValueName: "Path"; ValueData: "{app}\bin;{code:GetGTkDir|none}"; Check: AdminCheck(yes)

[Icons]
Name: "{group}\GIMP 2"; Filename: "{app}\bin\gimp-2.0.exe"; WorkingDir: "{code:GetGTkDir|none}"
Name: "{userdesktop}\GIMP 2"; Filename: "{app}\bin\gimp-2.0.exe"; WorkingDir: "{code:GetGTkDir|none}"; MinVersion: 4,4; Tasks: desktopicon
Name: "{userappdata}\Microsoft\Internet Explorer\Quick Launch\GIMP 2"; Filename: "{app}\bin\gimp-2.0.exe"; WorkingDir: "{code:GetGTkDir|none}"; MinVersion: 4,4; Tasks: quicklaunchicon

[Run]
;Filename: "{app}\bin\gimp.exe"; Description: "Launch The GIMP"; Flags: postinstall nowait skipifsilent
Filename: "{app}\bin\gimp-2.0.exe"; Description: "{code:GetText|LaunchGimp}"; Flags: postinstall nowait skipifsilent shellexec

[UninstallDelete]
Type: "files"; Name: "{app}\gimp.url"
;Type: "files"; Name: "{app}\gimpwinfaq.url"
Type: "files"; Name: "{app}\lib\gimp\1.2\plug-ins\gif.exe"

[Messages]
WelcomeLabel1=Welcome to [name] Setup Wizard
AboutSetupNote=Setup built by Jernej Simoncic, jernej.simoncic@guest.arnes.si

[Code]
program Setup;

  type
  DWORD = Cardinal;
  TSystemInfo = record
    wProcessorArchitecture: Word;
    wReserved: Word;
    dwPageSize: DWORD;
    lpMinimumApplicationAddress: Integer;
    lpMaximumApplicationAddress: Integer;
    dwActiveProcessorMask: DWORD;
    dwNumberOfProcessors: DWORD;
    dwProcessorType: DWORD;
    dwAllocationGranularity: DWORD;
    wProcessorLevel: Word;
    wProcessorRevision: Word;
  end;
  TAssocInfo = record
  	Description: String;
  	Extensions: TArrayOfString;
  	Selected: Boolean;
  end;

  //functions to get BPP & resolution
  function DeleteDC(hDC: Integer): Integer; external 'DeleteDC@GDI32 stdcall';
  function CreateDC(lpDriverName, lpDeviceName, lpOutput: String; lpInitData: Integer): Integer; external 'CreateDCA@GDI32 stdcall';
  function GetDeviceCaps(hDC, nIndex: Integer): Integer; external 'GetDeviceCaps@GDI32 stdcall';

  function GetSysColor(nIndex: Integer): Integer; external 'GetSysColor@user32 stdcall';

	procedure GetSystemInfo(var lpSystemInfo: TSystemInfo); external 'GetSystemInfo@kernel32.dll stdcall';
	
	function TerminateProcess(hProcess, uExitCode: Integer): Integer; external 'TerminateProcess@kernel32.dll stdcall';

  Const 
    GTkRegPath = 'SOFTWARE\GTK\2.0';
    RegKey = 'Inno Setup: App Path';

    RTFHeader = '{\rtf1\ansi\ansicpg1250\deff0\deflang1060{\fonttbl{\f0\fswiss\fprq2\fcharset238 Tahoma;}{\f1\fswiss\fprq2\fcharset0 Tahoma;}{\f2\fnil\fcharset2 Symbol;}}\viewkind4\uc1\pard{\*\pn\pnlvlblt\pnf2\pnindent0}\fi-284\li284\f0\fs16\sa60'#13#10;
    RTFHeaderM= '{\rtf1\deff0{\fonttbl{\f0\fswiss\fprq2\fcharset0 Tahoma;}}\viewkind4\uc1\fs16';
    //RTFHeaderM= '{\rtf1';
    RTFBullet = '{\pntext\f1\''B7\tab}';
    RTFPara   = '\par'#13#10;

    HORZRES = 8;
    BITSPIXEL = 12;
    PLANES = 14;
    
  var
    Associations: Array of TAssocInfo;
    Key: String;
    DataDir: String;
    ext: String;
    Warnings: Integer;
    GTkDir, GTKVer: String;
    numAssoc: Integer;
    ReadyMemoRichText: String;
    BadDLLs: TArrayOfString;
    BadDLLText: String;
    ShouldRenameBadDLLs: Integer;
    
	  lblAssocInfo2: TLabel;	
    lblDescriptionText: TLabel;

  function Replace(SearchFor, ReplaceWith, Text: String): String;
  var t1,t2: String;
      i: Integer;
  begin
    i := 1;
    if Pos(SearchFor,Text)>0 then
    begin
      while i<Length(Text) do
      begin
        if Copy(Text,i,Length(SearchFor)) = SearchFor then
        begin
          t1 := Copy(Text,1,i-1);
          t2 := Copy(Text,i+Length(SearchFor),Length(Text));
          Text := t1 + ReplaceWith + t2;
          i := i + Length(ReplaceWith);
        end else
          i := i + 1;

      end;

    end;
    Result := Text;
  end;


	function Count(What, Where: String): Integer;
	//var tmp: String;
	begin
		Result := 0;
		if Length(What) = 0 then
			exit; //avoid infinite loop
		while Pos(What,Where)>0 do
		begin
			Where := Copy(Where,Pos(What,Where)+Length(What),Length(Where));
			Result := Result + 1;
		end;
	end;
	

	//convert comma-separated text into array
	procedure Explode(var Dest: TArrayOfString; Text: String; Separator: String);
	var tmp: Integer;
	begin
		SetArrayLength(Dest,Count(Separator,Text)+1)

	  tmp := 0;
    repeat
      if Pos(Separator,Text)>0 then
      begin
        Dest[tmp] := Copy(Text,1,Pos(Separator,Text)-1)
 			  Text := Copy(Text,Pos(Separator,Text)+Length(Separator),Length(Text));
        tmp := tmp + Length(Separator);
      end else
      begin
         Dest[tmp] := Text;
         Text := '';
      end;
	  until Length(Text)=0;
	end;


  //ConstList: list of constants
  //Replacements: list of replacements, in the same order as constants; both lists are strings,
  //              with elements separated by #1
  //Text: text to process
  // Example: ProcessConstants('{foo}'#1'{bar}','BAR'#1'FOOBAR','bla{foo}bla{bar}bla') returns
  // 'blaBARblaFOOBARbla'
  function ProcessConstants(ConstList, Replacements, Text: String): String;
  var	Consts, Replac: TArrayOfString;
  		i: Integer;
  begin
  	Explode(Consts,ConstList,#1);
  	Explode(Replac,Replacements,#1);
 		Result := Text;
  	if GetArrayLength(Consts) <> GetArrayLength(Replac) then
  		exit;

  	for i := 0 to GetArrayLength(Consts) - 1 do
  		Result := Replace(Consts[i],Replac[i],Result);
  end;


  function GetText(ID: String): String;
  var LangFile: String;
  begin
  	LangFile := ExpandConstant('{tmp}\'+ActiveLanguage+'.isl+');

  	if not FileExists(LangFile) then
  		ExtractTemporaryFile(ActiveLanguage+'.isl+');

  	Result := GetIniString('Gimp',ID,'<string missing from language file>',LangFile);
  	Result := Replace('%n',#13#10,Result);

  end;

  function GetCPU(): Word;
	var
  	SysInfo: TSystemInfo;
  	S: String;
  begin
	  GetSystemInfo(SysInfo);
  	{S := 'Arch:' + IntToStr(SysInfo.wProcessorArchitecture) + '// PageSize:' + IntToStr(SysInfo.dwPageSize)
  	     + '// actCPUmask:' + IntToStr(SysInfo.dwActiveProcessorMask) + '// num cpu:' +
  	     IntToStr(SysInfo.dwNumberOfProcessors) + '// CPU type:' + IntToStr(SysInfo.dwProcessorType) +
  	     '// Alloc.Gran:' + IntToStr(SysInfo.dwAllocationGranularity) + '// cpu level:' +
  	     IntToStr(SysInfo.wProcessorLevel) + '// cpu rev:' + intToStr(SysInfo.wProcessorRevision);
  	     
  	MsgBox(S,mbInformation,0);}
  	
		Result := SysInfo.wProcessorLevel;
	end;


  function InitializeSetup(): Boolean;
  var i, j, xres, bpp, pl, tmp, rescode: Integer;
      SetupINI, S, S1, S2: String;
    	BadDLLList: String;
      hDC: Integer;
      PathDir: String;
      PathDirs: TArrayOfString;
  begin
    Result := True;

    If not RegQueryStringValue(HKLM, GTkRegPath, 'Path', GTkDir) then //find GTK+
      RegQueryStringValue(HKCU, GTkRegPath, 'Path', GTkDir);          //install dir

    If not RegQueryStringValue(HKLM, GTkRegPath, 'Version', GTKVer) then //GTK+ version
      RegQueryStringValue(HKCU, GTkRegPath, 'Version', GTKVer);

    //File associations, bad DLL list
 		if not ExtractTemporaryFile('setup.ini') then
		begin
			MsgBox(GetText('ErrorExtractingTemp'),mbError,0);
			Result := False;
			exit;
		end;
		SetupINI := ExpandConstant('{tmp}\setup.ini');
		numAssoc := GetIniInt('File Associations','Total',0,0,0,SetupINI);
		SetArrayLength(Associations,numAssoc);

		// Try to find the settings that were stored last time
		for i := 0 to NumAssoc - 1 do
		begin
			Associations[i].Description := GetIniString('File Associations',IntToStr(i+1),'<no description>',SetupINI);
			ext := GetIniString('File Associations',IntToStr(i+1)+'X','<undefined>',SetupINI);

			Explode(Associations[i].Extensions, ext, ',');
		
			ext:='Associate:' + Associations[i].Extensions[0];
			if GetPreviousData(ext,'0') = '0' then
				Associations[i].Selected := False
			else
				Associations[i].Selected := True;
		end;

    //get resolution & BPP
    hDC := CreateDC('DISPLAY', '', '', 0);
    pl := GetDeviceCaps(hDC, PLANES);
    bpp := GetDeviceCaps(hDC, BITSPIXEL);
    xres := GetDeviceCaps(hDC, HORZRES);
    tmp := DeleteDC(hDC);
    tmp := pl * bpp;
    
    if tmp<16 then //color depth
      Warnings:=Warnings Or 2;

    if xres<1024 then //resolution
      Warnings:=Warnings Or 4;

    if IsAdminLoggedOn=False then //admin
      Warnings:=Warnings Or 8;

    {If IsAdminLoggedOn=False then
      If RegValueExists(HKLM, GTkRegPath, 'Path') then //GTK+ was installed by admin but user isn't admin
      begin
        Warnings:=Warnings or 64;
        Warnings:=Warnings or 1;
      end;}

    If (GTkDir='') then //GTK was not found
    begin

      Warnings:=Warnings Or 16;
      Warnings:=Warnings Or 1;

    end else
    begin //detect if the libraries are in bin (my installer) or lib (dropline) subdir
    
      if FileExists(GTKDir + '\bin\libgtk-win32-2.0-0.dll') then
        GTKDir := GTKDir + '\bin'
      else if FileExists(GTKDir + '\lib\libgtk-win32-2.0-0.dll') then
        GTKDir := GTKDir + '\lib'
      else
      begin
        Warnings:=Warnings Or 16; //neither, something is wrong
        Warnings:=Warnings Or 1;
      end;

      //check GTK+ version
      if (GTKDir <> '') and (GTKVer < '2.2.4-20040124') then
      begin
        //workaround for Dropline installer, which doesn't set the revision and cvsdate (TODO: should warn)
        if not (FileExists(GTKDir + '\libfontconfig-1.dll') and FileExists(GTKDir + '\freetype6.dll') and FileExists(GTKDir + '\libpangoft2-1.0-0.dll')) then
        begin
          Warnings:=Warnings Or 128;
          Warnings:=Warnings Or 1;
        end;
      end;
      if not FileExists(GTKDir + '\libfontconfig-1.dll') then
        Warnings := Warnings or 1025;
      if not FileExists(GTKDir + '\freetype6.dll') then
        Warnings := Warnings or 1025;
      if not FileExists(GTKDir + '\libpangoft2-1.0-0.dll') then
        Warnings := Warnings or 1025;

    end;

		
    //locate potentionally problematic DLL files in system search path
    S := GetIniString('Setup','DLLList','<none>',SetupINI);
		Explode(BadDLLs, S, ',');

		PathDir := ExpandConstant('{sys}')+';'+ExpandConstant('{win}')+';'+ExpandConstant('{%PATH}');
		Explode(PathDirs, PathDir, ';');

		S := '';
		for i := 0 to GetArrayLength(PathDirs) - 1 do
			if (GTKDir <> '') and (LowerCase(GetShortName(PathDirs[i])) = LowerCase(GetShortName(GTKDir))) then
			begin
			  if GTKDir <> '' then //directories that come after GTK+ install dir shouldn't matter
			    break;
      end else			
				for j := 0 to GetArrayLength(BadDLLs) - 1 do
					if FileExists(AddBackslash(PathDirs[i])+BadDLLs[j]) then
						if Pos(BadDLLs[j],S) = 0 then
							S := S + AddBackslash(PathDirs[i])+BadDLLs[j] + ',';
							
		if Length(S) > 0 then
		begin
			S := Copy(S,1,Length(S)-1);
			Explode(BadDLLs, S, ',');
		end else
			SetArrayLength(BadDLLs,0);

		BadDLLList := '';
		for i := 0 to GetArrayLength(BadDLLs) - 1 do
			BadDLLList := BadDLLList + BadDLLs[i] + '//';
    BadDLLList := Copy(BadDLLList,1,Length(BadDLLList) - 2);
        
    BadDLLList := Replace('\','\\',BadDLLList);
    BadDLLList := Replace('//',RTFPara,BadDLLList);
    BadDLLList := '\pard\li284 ' + BadDLLList + '\sa60 ';

    S1 := '{p}'#1'{dlllist}'#1'{b}'#1'{/b}'#1'{i}'#1'{/i}';
    S2 := RTFPara+#1+BadDLLList+RTFPara+'\pard '#1'\b '#1'\b0 '#1'\i '#1'\i0 ';

		BadDLLText := RTFHeaderM + '\sa60';
 		S := GetText('DLLPageMessage');
 		BadDLLText := BadDLLText + ProcessConstants(S1,S2,S) + RTFPara;
 		BadDLLText := BadDLLText + #13#10'}';

    if not UsingWinNT() then
      Warnings := Warnings or 256;
      
    //detect CPU
    If (GetCPU < 5) then
      Warnings := Warnings or 513;
      
    for i := 1 to ParamCount do
      if Pos('/ignoreerrors',Lowercase(ParamStr(i))) > 0 then
        if (Warnings and 1) = 1 then
          Warnings := Warnings Xor 1;

    #ifdef testing
    //Warnings := 65534;
    #endif

    #ifdef cvsdate
    MsgBox(ProcessConstants('{cvsdate}','{#=cvsdate}',GetText('BuiltFromCVS')),mbInformation,0);
    #endif

  end;


  procedure RegisterPreviousData(PreviousDataKey: Integer);
  var i: integer;
      CurSubPage: Integer;
      ext: string;
  begin
    //Save informations about selected associations
    for i := 0 to NumAssoc - 1 do
    begin
    	ext:='Associate:' + Associations[i].Extensions[0];
      if Associations[i].Selected then
      	SetPreviousData(PreviousDataKey, ext, '1')
      else
      	SetPreviousData(PreviousDataKey, ext, '0');
    end;
  end;
  
  
  procedure TerminateSetup(Sender: TObject);
  begin
    PostMessage(StrToInt(ExpandConstant('{wizardhwnd}')),18,0,0);
  end;


  procedure AssociationClicked(Sender: TObject);
  var i,j: Integer;
  begin
  	//MsgBox('OnClickCheck',mbInformation,0);
  	for i := 0 to numAssoc - 1 do
  	  if TNewCheckListbox(Sender).Selected[i] then
  	  begin
  	    ext := '';
  	    for j := 0 to GetArrayLength(Associations[i].Extensions) - 1 do
  	      ext := ext + LowerCase(Associations[i].Extensions[j]) + ', ';
  	    ext := Copy(ext, 1, Length(ext) - 2);
  	    
  	    lblAssocInfo2.Caption := #13+GetText('SelectAssociationsExtensions')+' ' + ext {+ #13 + Associations[i].Description};
    	end;
  end;
  
  
  procedure EnableNextButton(Sender: TObject);
  begin
    WizardForm.NextButton.Enabled := True;
  end;
  

  function ScriptDlgPages(CurPage: Integer; BackClicked: Boolean): Boolean;
  var CurSubPage,i: Integer;
      Next: Boolean;
      Text, BadDLLList: String;
      S,S1,S2: String;
      clBtnFace,clBtnShadow,clBtnLight,clText,clGrayText,clHilight,clHilightText,
      clTooltip,clTooltipText,clWindow,clWindowText: String;
      clHex: String;
  		RichEditViewer: TRichEditViewer;
  		rbYes, rbNo: TRadioButton;
		  lblAssocInfo: TLabel;
		  clbAssociations: TNewCheckListBox;
  begin

    if (not BackClicked and (CurPage = wpSelectProgramGroup)) or (BackClicked and (CurPage = wpSelectTasks)) then
    begin
      ///////////////////////////
      // File Association page //
      ///////////////////////////

      if not BackClicked then
        CurSubPage := 0
      else
        CurSubPage := 1;
        
      while (CurSubPage>=0) and (CurSubPage<=1) and not Terminated do
      begin
        case CurSubPage of
          0:
          begin

            ScriptDlgPageOpen();
            
            if IsAdminLoggedOn=False then //no associations for non-admins :(
            begin
            
              ScriptDlgPageSetCaption(GetText('SelectAssociationsCaption'));
              ScriptDlgPageSetSubCaption1(GetText('SelectAssociationsSubcaption'));

              Text := GetText('SelectAssociationsAdminRights');

              Next := OutputMsg(Text,True);
              
            end else begin

              ScriptDlgPageSetCaption(GetText('SelectAssociationsCaption'));
              ScriptDlgPageSetSubCaption1(GetText('SelectAssociationsSubcaption'));

							{ lblAssocInfo }
							lblAssocInfo := TLabel.Create(WizardForm.ScriptDlgPanel);
							lblAssocInfo.Parent := WizardForm.ScriptDlgPanel;
							lblAssocInfo.Left := 0;
							lblAssocInfo.Top := 0;
							lblAssocInfo.Width := WizardForm.ScriptDlgPanel.Width;
							lblAssocInfo.Height := 13;
							lblAssocInfo.Caption := GetText('SelectAssociationsInfo1')+#13+GetText('SelectAssociationsInfo2')+#13;
							
							{ lblAssocInfo2 }
							lblAssocInfo2 := TLabel.Create(WizardForm.ScriptDlgPanel);
							lblAssocInfo2.Parent := WizardForm.ScriptDlgPanel;
							lblAssocInfo2.Left := 0;
							lblAssocInfo2.Width := WizardForm.ScriptDlgPanel.Width;
							lblAssocInfo2.Height := 13;
							lblAssocInfo2.AutoSize := True;
							lblAssocInfo2.Caption := #13+GetText('SelectAssociationsExtensions');
							
							{ clbAssociations }
							clbAssociations := TNewCheckListBox.Create(WizardForm.ScriptDlgPanel);
							with clbAssociations do
							begin
								Parent := WizardForm.ScriptDlgPanel;
								Left := 0;
								Top := lblAssocInfo.Top + lblAssocInfo.Height;
								Width := WizardForm.ScriptDlgPanel.Width;
								Height := WizardForm.ScriptDlgPanel.Height - lblAssocInfo.Height - lblAssocInfo2.Height - 8;
								Style := lbOwnerDrawVariable;
								TabOrder := 1;
								Flat := True;

								lblAssocInfo2.Top := Height + Top;

								OnClick := @AssociationClicked;

								for i := 0 to numAssoc - 1 do
									AddCheckBox(Associations[i].Description,'',0,Associations[i].Selected,True,False,nil);								

							end;							

							Next := ScriptDlgPageProcessCustom();

							for i := 0 to numAssoc - 1 do							
								Associations[i].Selected := clbAssociations.Checked[i];								
								
            end;

          end;
          1:
          begin

            ScriptDlgPageOpen();
            ScriptDlgPageSetCaption(GetText('SupportedFilesInfoCaption'));
            ScriptDlgPageSetSubCaption1(GetText('SupportedFilesInfoSubcaption'));
            
            Text := '';
            Text := GetText('SupportedFilesInfoPS')+#13#13;

            Next := OutputMsg(Text,True);

          end;
        end;

        if Next then
          CurSubPage := CurSubPage + 1
        else
          CurSubPage := CurSubPage - 1;
      end;

      if not BackClicked then
        Result := Next
      else
        Result := not Next;

      ScriptDlgPageClose(not Result);

    end else if (not BackClicked and (CurPage = {wpInfoBefore} wpLicense)) or (BackClicked and (CurPage = wpSelectDir)) then
    begin
      ////////////////////////////////////////
      // Warnings (color depth, resolution) //
      ////////////////////////////////////////
      If Warnings > 0 then begin //display the page only if necessary
        ScriptDlgPageOpen();
        ScriptDlgPageSetCaption(GetText('WarningsCaption'));
        ScriptDlgPageSetSubCaption1(GetText('WarningsSubcaption'));

        Text := RTFHeader;
        if (Warnings and 1024)=1024 then
          Text := Text+RTFBullet+GetText('WarningsIncompleteGTK')+RTFPara;
        if (Warnings and 512)=512 then
          Text := Text+RTFBullet+GetText('WarningsPentium')+RTFPara;
        if (Warnings and 256)=256 then
          Text := Text+RTFBullet+GetText('WarningsWin9xME')+RTFPara;
        if (Warnings and 128)=128 then
          Text := Text+RTFBullet+GetText('WarningsOldGTK')+RTFPara;
        if (Warnings and 64)=64 then
          Text := Text+RTFBullet+GetText('WarningsGTKAdmin')+RTFPara;
        if (Warnings and 16)=16 then
          Text := Text+RTFBullet+GetText('WarningsNoGTK')+RTFPara;
        If (Warnings and 2)=2 then
          Text := Text+RTFBullet+GetText('WarningsColorMode')+RTFPara;
        If (Warnings and 4)=4 then
          Text := Text+RTFBullet+GetText('WarningsResolution')+RTFPara;
        if (Warnings and 8)=8 then
          Text := Text+RTFBullet+GetText('WarningsAdmin')+RTFPara;

        if (Warnings and 1)=1 then
        begin
          Text := Text+'\pard\par\b\f0 '+GetText('WarningsCantContinue')+'\b0\f1\par'#13#10;
          WizardForm.BackButton.Enabled := False;
          WizardForm.NextButton.Enabled := False;
          WizardForm.CancelButton.OnClick := @TerminateSetup;
          WizardForm.CancelButton.Caption := GetText('ButtonClose');
        end;

        Text := Text + '}';

        {
        if (Warnings and 1)<>1 then
          Next := OutputMsg(Text,True)
        else
        begin
          Next := OutputMsg(Text,False);
          exit;
        end;
        }

        RichEditViewer := TRichEditViewer.Create(WizardForm.ScriptDlgPanel);
        with RichEditViewer do
        begin
          Parent := WizardForm.ScriptDlgPanel;
          Scrollbars := ssVertical;
          UseRichEdit := True;
          ReadOnly := True;
          Width := WizardForm.ScriptDlgPanel.Width;
          Height := WizardForm.ScriptDlgPanel.Height - 8;
        end;
        RichEditViewer.RTFText := Text;

        Next := ScriptDlgPageProcessCustom();

        if not BackClicked then
          Result := Next
        else
          Result := not Next;

        ScriptDlgPageClose(not Result);
        
      end else begin
        Result := True;
      end;

    end else if (not BackClicked and (CurPage = wpSelectTasks)) or (BackClicked and (CurPage = wpReady)) then
    begin

    	if GetArrayLength(BadDLLs) > 0 then
    	begin

        ScriptDlgPageOpen();
        ScriptDlgPageSetCaption(GetText('DLLPageCaption'));
        ScriptDlgPageSetSubCaption1(GetText('DLLPageSubcaption'));

        RichEditViewer := TRichEditViewer.Create(WizardForm.ScriptDlgPanel);
        with RichEditViewer do
        begin
          Parent := WizardForm.ScriptDlgPanel;
          Scrollbars := ssVertical;
          UseRichEdit := True;
          ReadOnly := True;
          Top := 0;
          Left := 0;
          Width := WizardForm.ScriptDlgPanel.Width;
          Height := WizardForm.LicenseMemo.Height + WizardForm.LicenseMemo.Top;
          RTFText := BadDLLText;
        end;

        rbYes := TRadioButton.Create(WizardForm.ScriptDlgPanel);
        with rbYes do
        begin
        	Parent := WizardForm.ScriptDlgPanel;
        	Left := WizardForm.LicenseAcceptedRadio.Left - WizardForm.ScriptDlgPanel.Left;
        	Width := WizardForm.ScriptDlgPanel.Width - Left;
        	Top := WizardForm.LicenseAcceptedRadio.Top - WizardForm.ScriptDlgPanel.Top;
        	Height := WizardForm.LicenseAcceptedRadio.Height;
        	Caption := GetText('DLLPageOptionRename');
        	OnClick := @EnableNextButton;
        	if ShouldRenameBadDLLs = 1 then
        		Checked := True;
        end;

        rbNo := TRadioButton.Create(WizardForm.ScriptDlgPanel);
        with rbNo do
        begin
        	Parent := WizardForm.ScriptDlgPanel;
        	Left := WizardForm.LicenseNotAcceptedRadio.Left - WizardForm.ScriptDlgPanel.Left;
        	Width := WizardForm.ScriptDlgPanel.Width - Left;
        	Top := WizardForm.LicenseNotAcceptedRadio.Top - WizardForm.ScriptDlgPanel.Top;
        	Height := WizardForm.LicenseNotAcceptedRadio.Height;
        	Caption := GetText('DLLPageOptionSkip');
        	OnClick := @EnableNextButton;
        	if ShouldRenameBadDLLs = 2 then
        		Checked := True;
        end;
        
       	if ShouldRenameBadDLLs = 0 then
	        WizardForm.NextButton.Enabled := False;

        Next := ScriptDlgPageProcessCustom();

       	if rbYes.Checked then
       		ShouldRenameBadDLLs := 1
       	else if rbNo.Checked then
					ShouldRenameBadDLLs := 2;

        if not BackClicked then
          Result := Next
        else
          Result := not Next;

        ScriptDlgPageClose(not Result);

    	end else
    	begin	
    		Result := True;
    	end;
    	 
    end

    (*end else if CurPage = wpFinished then

    //GTK+ installer will take care of this in the future
    ////////////////////////////////
    // save color scheme to gtkrc //
    ////////////////////////////////
    begin

      GTKRC := ExpandConstant('{app}\etc\gimp\1.2\gtkrc_user');
      //GTKRC := GetGTkDir('')+'\etc\gimp\1.2\gtkrc_user';

      if (ShouldProcessEntry('base','gtkrc')=srYes) then
      begin
        if not FileExists(GTKRC) then
          MsgBox('Error: Windows color scheme could not be saved.',0,0)
        else
        begin
        
          //get the windows' colors
          clBtnFace := RGBBGR(GetSysColor(15));
          clBtnShadow := RGBBGR(GetSysColor(16));
          clBtnLight := RGBBGR(GetSysColor(20));
          clText := RGBBGR(GetSysColor(18));
          clWindow := RGBBGR(GetSysColor(5));
          clWindowText := RGBBGR(GetSysColor(8));
          clGrayText := RGBBGR(GetSysColor(17));
          clHilight := RGBBGR(GetSysColor(13));
          clHilightText := RGBBGR(GetSysColor(14));
          clTooltip := RGBBGR(GetSysColor(24));
          clTooltipText := RGBBGR(GetSysColor(23));
          
          
          //generate the gtkrc file
          S:='style "default"';
          S:=S+#13'{';
          S:=S+#13'  bg[NORMAL] = "#'+clBtnFace+'"';
          S:=S+#13'  bg[SELECTED] = "#'+clHilight+'"';
          S:=S+#13'  bg[INSENSITIVE] = "#'+clBtnFace+'"';
          S:=S+#13'  bg[ACTIVE] = "#'+clBtnFace+'"';
          S:=S+#13'  bg[PRELIGHT] = "#'+clHilight+'"';
          S:=S+#13'';
          S:=S+#13'  base[NORMAL] = "#'+clWindow+'"';
          S:=S+#13'  base[SELECTED] = "#'+clHilight+'"';
          S:=S+#13'  base[INSENSITIVE] = "#'+clBtnFace+'"';
          S:=S+#13'  base[ACTIVE] = "#'+clWindow+'"';
          S:=S+#13'  base[PRELIGHT] = "#'+clWindow+'"';
          S:=S+#13'';
          S:=S+#13'  text[NORMAL] = "#'+clHilightText+'"';
          S:=S+#13'  text[SELECTED] = "#'+clText+'"';
          S:=S+#13'  text[INSENSITIVE] = "#'+clBtnFace+'"';
          S:=S+#13'  text[ACTIVE] = "#'+clHilightText+'"';
          S:=S+#13'  text[PRELIGHT] = "#'+clHilightText+'"';
          S:=S+#13'';
          S:=S+#13'  fg[NORMAL] = "#'+clText+'"';
          S:=S+#13'  fg[SELECTED] = "#'+clText+'"';
          S:=S+#13'  fg[INSENSITIVE] = "#'+clGrayText+'"';
          S:=S+#13'  fg[ACTIVE] = "#'+clText+'"';
          S:=S+#13'  fg[PRELIGHT] = "#'+clText+'"';
          S:=S+#13'}';
          S:=S+#13'class "*" style "default"';
          S:=S+#13'';
          S:=S+#13'style "tooltip"';
          S:=S+#13'{';
          S:=S+#13'  bg[NORMAL] = "#'+clTooltip+'"';
          S:=S+#13'  fg[NORMAL] = "#'+clTooltipText+'"';
          S:=S+#13'}';
          S:=S+#13'widget "*gtk-tooltip*" style "tooltip"';
          S:=S+#13'';
          S:=S+#13'style "button" {';
          S:=S+#13'  bg[NORMAL]    = "#'+clBtnFace+'"';
          S:=S+#13'  bg[PRELIGHT]  = "#'+clBtnFace+'"';
          S:=S+#13'  bg[INSENSITIVE] = "#'+clBtnFace+'"';
          //S:=S+#13'  fg[PRELIGHT] = "#'+clHilight+'"';
          S:=S+#13'}';
          S:=S+#13'widget "*Gtk*Button*" style "button"';
          S:=S+#13'';
          S:=S+#13'style "text" {';
          S:=S+#13'  fg[NORMAL] = "#'+clWindowText+'"';
          S:=S+#13'}';
          S:=S+#13'widget_class "*Entry*" style "text"';
          S:=S+#13'widget_class "*Tree*" style "text"';
          S:=S+#13'widget_class "*Text*" style "text"';
          S:=S+#13'widget_class "*GtkSpinButton*" style "text"';
          S:=S+#13'widget_class "*Editable*" style "text"';
          S:=S+#13'widget_class "*Combo*" style "text"';
          S:=S+#13'';

          if not SaveStringToFile(GTKRC,S,False) then
            MsgBox('Error: could not update color scheme file.',0,0);

        end;
      end;
      
      Result := True;
    *)
    {end else if CurPage = wpSelectDir then
    begin
    

      if Pos(' ',WizardDirValue)>0 then
      begin
      
        S := 'Due to a problem with some versions of GLib, The Gimp 2.0 should not be installed to a path ';
        S := S + 'which contains spaces, as it may fail to load plugins.';
        S := S + #13#13'Do you wish to continue anyway?';
        
        if MsgBox(S,mbError,MB_YESNO) = IDYES then
          Result := True
        else
          Result := False;
          
      end else
        Result := True;


    end else if curPage = wpSelectTasks then
    begin
      if not BackClicked then
      begin
      	WizardForm.ReadyMemo.Left := -(WizardForm.ReadyMemo.Width+10);
      	
    	end;
    	Result := True;
    end} else
    begin
      Result := True;
    end;
  end;


  function ParseReadyMemoText(Spaces,Text: String): String;
  var t: String;
  begin
    t := Copy(Text,Pos(#10,Text)+1,Length(Text));
    t := Replace('\','\\',t);
    t := Replace(#13#10,'\par'#13#10,t);
    t := Replace(Spaces,'',t);
    Result := '\b ' + Copy(Text,1,Pos(#13,Text)-1) + '\par' +
              '\li284\b0 ' + t + '\par'#13#10'\pard\par'#13#10;
  end;


  function UpdateReadyMemo(Space, NewLine, MemoUserInfo, MemoDirInfo, MemoTypeInfo, MemoComponentsInfo, MemoGroupInfo, MemoTasksInfo: String): String;
  var S: String;
      ShowAssoc: Boolean;
      i,j: Integer;
  begin

    (* Fill the 'Ready Memo' with setup information *)
    
    {S := MemoDirInfo + NewLine + NewLine;                //installation directory

    S := S + MemoTypeInfo + NewLine + NewLine;           //installation type (full/compact/custom)

    S := S + MemoComponentsInfo + NewLine + NewLine;     //selected components}
    
    S := RTFHeaderM;
    S := S + ParseReadyMemoText(Space,MemoDirInfo);
    S := S + ParseReadyMemoText(Space,MemoTypeInfo);
    S := S + ParseReadyMemoText(Space,MemoComponentsInfo);

    for i := 0 to NumAssoc - 1 do
      if Associations[i].Selected then
        ShowAssoc:=True;

    if ShowAssoc then
    begin
      S := S + '\b '+GetText('ReadyMemoAssociations')+'\par'#13#10'\li284\b0 '; //which file types to associate

      (*for i := 0 to NumAssoc do
        if AssocValues[i]='1' then
          S := S + NewLine + Space + AssocPrompts[i];*)
  		for i := 0 to numAssoc - 1 do							
		    if Associations[i].Selected then
		      for j := 0 to GetArrayLength(Associations[i].Extensions) - 1 do
		        S := S + LowerCase(Associations[i].Extensions[j]) + ', ';
		        
		  S := Copy(S, 1, Length(S) - 2) + '\par'#13#10'\pard\par'#13#10;

      //S := S + NewLine + NewLine;
    end;
    
    //S := S + MemoGroupInfo;                              //program group
    S := S + ParseReadyMemoText(Space,MemoGroupInfo);

    If MemoTasksInfo<>'' then
      S := S + ParseReadyMemoText(Space,MemoTasksInfo);
      //S := S + NewLine + NewLine + MemoTasksInfo;        //additional icons
      
    ReadyMemoRichText := Copy(S,1,Length(S)-11) + '}';
    Result := 'If you see this, I screwed up';
  end;


  function NextButtonClick(CurPage: Integer): Boolean;
  var ResCode: Integer;
  begin
    Result := ScriptDlgPages(CurPage, False);
  end;


  function BackButtonClick(CurPage: Integer): Boolean;
  begin
    Result := ScriptDlgPages(CurPage, True);
  end;


  //GTK+ installation directory
  function GetGTkDir(S: String): String;
  begin
    Result := GTKdir;
  end;


  //Check, if file association should be registered.
  (*function AssocCheck(assoc: String): Boolean;
  var an,i: Integer;
  begin

    if IsAdminLoggedOn=false then //don't trigger warning dialogs if admin isn't logged on
      Result:=False
    else
    begin
    
      for i := 0 to NumAssoc do
      begin
        {If Pos(assoc,AssocPrompts[i])>0 then
        begin
          If AssocValues[i]='1' then
            Result := True
          else
            Result := False;
        end;}
      end;

    end;

  end;*)
  
  
  procedure ComponentsListOnClick(Sender: TObject);
  var i: Integer;
  begin
    for i := 0 to 4 do
      if TNewCheckListBox(Sender).Selected[i] then
      begin
        case i of
        0: lblDescriptionText.Caption := GetText('ComponentsBaseDescription');
        1: lblDescriptionText.Caption := GetText('ComponentsTranslationsDescription');
        2: lblDescriptionText.Caption := GetText('ComponentsFreeTypeDescription');
        3: lblDescriptionText.Caption := GetText('ComponentsHelpDescription');
        4: lblDescriptionText.Caption := GetText('ComponentsGAPDescription');
        end;
        exit;
      end;
  end;
  
  procedure CurPageChanged(CurPage: Integer);
  var NewReadyMemo: TRichEditViewer;
      pnlDescription: TPanel;
      lblDescription: TNewStaticText;
  begin
    case curPage of
    wpSelectComponents:
      begin
      	if WizardForm.ComponentsList.Width = WizardForm.ScriptDlgPanel.Width then
	        WizardForm.ComponentsList.Width := Round(WizardForm.ComponentsList.Width * 0.6);
        
        WizardForm.ComponentsList.OnClick := @ComponentsListOnClick;
        
        pnlDescription := TPanel.Create(WizardForm.ComponentsList.Parent);
        with pnlDescription do
        begin
          Parent := WizardForm.ComponentsList.Parent;
          Left := WizardForm.ComponentsList.Left + WizardForm.ComponentsList.Width + 8;
          Width := WizardForm.TypesCombo.Width - WizardForm.ComponentsList.Width - 8;
          BevelOuter := bvLowered;
          BevelInner := bvRaised;
          Top := WizardForm.ComponentsList.Top + 5;
          Height := WizardForm.ComponentsList.Height - 5;
        end;
        
        lblDescription := TNewStaticText.Create(WizardForm.ComponentsList.Parent)
        with lblDescription do
        begin
          Parent := WizardForm.ComponentsList.Parent;
          Left := pnlDescription.Left + 8;
          Top := WizardForm.ComponentsList.Top;
          AutoSize := True;
          Caption := GetText('ComponentsDescription');
        end;
        
        lblDescriptionText := TLabel.Create(pnlDescription);
        with lblDescriptionText do
        begin
          Parent := pnlDescription;
          Left := 8;
          WordWrap := True;
          AutoSize := False;
          Width := Parent.Width - 16;
          Height := Parent.Height - 20;
          Top := 12;
        end;
        
      end;
    wpReady:
      begin
        WizardForm.ReadyMemo.Visible := False;
      
        NewReadyMemo := TRichEditViewer.Create(WizardForm.ReadyMemo.Parent);
        NewReadyMemo.Parent := WizardForm.ReadyMemo.Parent;//WizardForm.ScriptDlgPanel
        NewReadyMemo.Scrollbars := ssVertical;
        NewReadyMemo.Color := WizardForm.Color;
        NewReadyMemo.UseRichEdit := True;
        NewReadyMemo.RTFText := ReadyMemoRichText;
        NewReadyMemo.ReadOnly := True;
        NewReadyMemo.Left := WizardForm.ReadyMemo.Left;
        NewReadyMemo.Top := WizardForm.ReadyMemo.Top;
        NewReadyMemo.Width := WizardForm.ReadyMemo.Width;
        NewReadyMemo.Height := WizardForm.ReadyMemo.Height;
        NewReadyMemo.Visible := True;
      
        {NewReadyMemo.Width := WizardForm.ScriptDlgPanel.Width;
        NewReadyMemo.Height := WizardForm.ScriptDlgPanel.Height;}
      end;
    end;
  end;

  function AdminCheck(S: String): Boolean;
  begin
    if S='yes' then
    begin
      if IsAdminLoggedOn then
        Result:=True
      else
        Result:=False;
    end else
    begin
      if IsAdminLoggedOn then
        Result:=False
      else
        Result:=True;
    end;
  end;


  procedure CurStepChanged(CurStep: Integer);
  var S,GTKRC: String;
      clBtnFace,clBtnShadow,clBtnLight,clText,clGrayText,clHilight,clHilightText,
      clTooltip,clTooltipText,clWindow,clWindowText: String;
      i: Integer;
  begin
    (* TODO: automatically uninstall previous Gimp version
    if curStep = csCopy then
      if OldGTKUninstall <> '' then
      begin
      
        WizardForm.StatusLabel.Caption := GetText('UninstallingOldGTK');

        if OldGTKUninstallParams = '' then
        	MsgBox(GetText('UninstallGTKManually'),mbInformation,0);
        
        if not InstExec(OldGTKUninstall,OldGTKUninstallParams,ExtractFilePath(OldGTKUninstall),True,False,0,i) then
        begin
          MsgBox(GetText('ErrorUninstallingOldGTK'),mbInformation,0);
          TerminateSetup2;
        end;
        
        Sleep(1000);

        //wait for the uninstaller to clean after itself
        if FileExists(OldGTKUninstall) then
        begin
        	for i := 0 to 20 do
        	begin
        		Sleep(250);
        		if not FileExists(OldGTKUninstall) then
        			break;
        	end;
        	if i > 20 then
        	begin
	          MsgBox(GetText('ErrorUninstallingOldGTK'),mbInformation,0);
  	        TerminateSetup2;
        	end;
        end;
          
      end;
    *)
  	if curStep = csFinished then
  	begin
  	
  	  //rename DLLs			
			if (ShouldRenameBadDLLs = 1) and (GetArrayLength(BadDLLs) > 0) then
			begin
 			  WizardForm.StatusLabel.Caption := GetText('RenamingFiles');
				for i := 0 to GetArrayLength(BadDLLs) - 1 do
				begin
  			  WizardForm.FileNameLabel.Caption := BadDLLs[i];
					if not RenameFile(BadDLLs[i],BadDLLs[i]+'.off') then
					  MsgBox(ProcessConstants('{dll}',BadDLLs[i],GetText('CouldNotRenameDLL')),mbInformation,0);
				end;
			end;
  	end;

  end;
  
  
  //Inno treats multiple components on file as OR, which is why a check function is used
  function ComponentCheck(Which: String): Boolean;
  begin
    //TODO: split multiple components
    if ShouldProcessEntry(Which,'') then
      Result := True
    else
      Result := False;
  end;
  
  
  function AssociateFiles(): Boolean;
  var i,j: Integer;
  begin
    Result := False;
    if IsAdminLoggedOn then
  		for i := 0 to numAssoc - 1 do							
		    if Associations[i].Selected then
		      for j := 0 to GetArrayLength(Associations[i].Extensions) - 1 do
		        RegWriteStringValue(HKCR, '.'+Associations[i].Extensions[j],'','TheGIMP20');
  end;

begin
end.

